<?php

global $MESS;
$PathInstall = str_replace("\\", DIRECTORY_SEPARATOR, __FILE__);
$PathInstall = substr($PathInstall, 0, strlen($PathInstall) - strlen(DIRECTORY_SEPARATOR.'index.php'));

IncludeModuleLangFile($PathInstall . DIRECTORY_SEPARATOR . 'install.php');

/**
 * Installer for CleanTalk module
 *
 * @author 	Cleantalk team <http://cleantalk.org>
 */
class cleantalk extends CModule {

    public $MODULE_ID = 'cleantalk';
    public $MODULE_VERSION;
    public $MODULE_VERSION_DATE;
    public $MODULE_NAME;
    public $MODULE_DESCRIPTION;
    public $MODULE_CSS;

    private $template_dir;
    private $template_file;
    private $system_template_dir;
    private $local_template_dir;
    private $local_compo_template_dir;
    private $pattern;
    private $ct_template_addon_tag;
    private $ct_template_addon_body;

    /**
     * Constructor
     */
    public function __construct() {
        global $DOCUMENT_ROOT;
        $arModuleVersion = array();

        $path = str_replace("\\", DIRECTORY_SEPARATOR, __FILE__);
        $path = substr($path, 0, strlen($path) - strlen(DIRECTORY_SEPARATOR . 'index.php'));
        include($path . DIRECTORY_SEPARATOR .'version.php');

        if (is_array($arModuleVersion) && array_key_exists('VERSION', $arModuleVersion)) {
            $this->MODULE_VERSION = $arModuleVersion['VERSION'];
            $this->MODULE_VERSION_DATE = $arModuleVersion['VERSION_DATE'];
        }

        $this->MODULE_NAME = GetMessage('CLEANTALK_MODULE_NAME');
        $this->MODULE_DESCRIPTION = GetMessage('CLEANTALK_MODULE_DESCRIPTION');

	// Values for all templates
	$this->ct_template_addon_tag = 'CLEANTALK template addon';
	$this->ct_template_addon_body = '
<?php
if(COption::GetOptionString( "cleantalk", "status" ) == "1"){
$_SESSION["ct_submit_register_time"] = time();
$ct_check_value = md5(rand(0, 1000));
$_SESSION["ct_check_key"] = $ct_check_value;
?>
<input type="hidden" name="ct_checkjs" id="ct_checkjs" value="0" />
<script type="text/javascript">
setTimeout("document.getElementById(\"ct_checkjs\").value = document.getElementById(\"ct_checkjs\").value.replace(\"0\", \"<?php echo $ct_check_value; ?>\");",1000);
</script>
<?php
}
?>
';

	// Values for system.auth.registration default template
	$this->SAR_template_dir  = '.default'; // without ending slash
	$this->SAR_template_file = 'template.php';
	//...with ending slash
	$this->SAR_system_template_dir = $DOCUMENT_ROOT.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'system.auth.registration'.DIRECTORY_SEPARATOR.'templates'.DIRECTORY_SEPARATOR;
	$this->SAR_local_template_dir = $DOCUMENT_ROOT.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'templates'.DIRECTORY_SEPARATOR.'.default'.DIRECTORY_SEPARATOR;
	$this->SAR_local_compo_template_dir = $this->SAR_local_template_dir.'components'.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'system.auth.registration'.DIRECTORY_SEPARATOR;
	$this->SAR_pattern = '/(<\?\/\/[\s\*]*\/User properties)/i';
    }

    public function DoInstall() {
        global $DOCUMENT_ROOT, $APPLICATION;
        if(!$this->InstallFiles())
	    return FALSE;
        if(!$this->InstallDB())
	    return FALSE;
        RegisterModule("cleantalk");
        $APPLICATION->IncludeAdminFile('Установка модуля CleanTalk', $DOCUMENT_ROOT.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'modules'.DIRECTORY_SEPARATOR.'cleantalk'.DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'step.php');
    }

    public function DoUninstall() {
        global $DOCUMENT_ROOT, $APPLICATION;
        UnRegisterModule("cleantalk");
        $this->UnInstallFiles();
        $this->UnInstallDB();
        $APPLICATION->IncludeAdminFile('Деинсталляция модуля CleanTalk', $DOCUMENT_ROOT.DIRECTORY_SEPARATOR.'bitrix'.DIRECTORY_SEPARATOR.'modules'.DIRECTORY_SEPARATOR.'cleantalk'.DIRECTORY_SEPARATOR.'install'.DIRECTORY_SEPARATOR.'unstep.php');
    }

    public function InstallFiles() {
	// Copy system.auth.registration default template from system dir to local dir and insert addon into
	$SAR_res = $this->install_ct_template($this->SAR_template_dir,
			$this->SAR_template_file,
			$this->SAR_system_template_dir,
			$this->SAR_local_template_dir,
			$this->SAR_local_compo_template_dir,
			$this->SAR_pattern,
			$this->ct_template_addon_tag,
			$this->ct_template_addon_body
	);

	// No more templates for now
	if($SAR_res == 0){
	    return TRUE;
	}else{
	    return FALSE;
	}
    }

    public function UnInstallFiles() {
	// Remove addon from local system.auth.registration default template
	$SAR_res = $this->uninstall_ct_template($this->SAR_template_dir,
			$this->SAR_template_file,
			$this->SAR_local_compo_template_dir,
			$this->ct_template_addon_tag
	);
    	return TRUE;	// always TRUE
    }

    public function InstallDB() {
	global $DB;

	$DB->Query('DROP TABLE IF EXISTS cleantalk_timelabels');
	if(!$DB->Query('CREATE TABLE cleantalk_timelabels ( ct_key varchar(255), ct_value int(11), PRIMARY KEY (ct_key))'))
	    return FALSE;
        RegisterModuleDependences('main', 'OnBeforeUserRegister', 'cleantalk', 'CleantalkClass', 'OnBeforeUserRegisterHandler');
        return TRUE;
    }

    public function UnInstallDB($arParams = Array()) {
	global $DB;

        UnRegisterModuleDependences('main', 'OnBeforeUserRegister', 'cleantalk', 'CleantalkClass', 'OnBeforeUserRegisterHandler');
	$rs = $DB->Query("DROP TABLE cleantalk_timelabels");
    }


    /**
     * Copies needed template from system dir to local dir and inserts CleanTalk addon into it
     *
     * @param 	&string $template_dir			Name of component's template dir (.default)
     * @param 	&string $template_file			Name of component's template file (template.php)
     * @param 	&string $system_template_dir		Full system dir of component templates (.../bitrix/components/bitrix/system.auth.registration/templates/)
     * @param 	&string $local_template_dir		Full local dir of templates (.../bitrix/templates/.default/)
     * @param 	&string $local_compo_template_dir	Full local dir of component template (.../bitrix/templates/.default/components/bitrix/system.auth.registration/)
     * @param 	&string $pattern			PCRE pattern to find place to insert CleanTalk addon before
     * @param 	&string $ct_template_addon_tag		Tag string to mark CleanTalk addon body
     * @param 	&string $ct_template_addon_body		HTML text of CleanTalk addon itself
     * @return 	int Returns error code or 0 when success
     */
    private function install_ct_template($template_dir,	// without ending slash
			     $template_file,
			     $system_template_dir,	// with ending slash
			     $local_template_dir,	// with ending slash
			     $local_compo_template_dir,	// with ending slash
			     $pattern,
			     $ct_template_addon_tag,
			     $ct_template_addon_body)
    {
	// Check system folders
	if(!file_exists($system_template_dir) || !file_exists($local_template_dir)){
		// No required system folders
		return 1;
	}

	//Check component templates folder
	if(!file_exists($local_compo_template_dir)){
		if(!mkdir($local_compo_template_dir, 0777, TRUE)){
			// Cannot create component templates folder
			return 2;
		}
	}

	// Check template subfulder in component templates folder - the Bitrix template itself
	if(!file_exists($local_compo_template_dir.$template_dir)){
		if(!CopyDirFiles($system_template_dir.$template_dir, $local_compo_template_dir.$template_dir, FALSE, TRUE)){
			// Cannot copy template of conponent from system folder to local folder
			return 3;
		}
	}

	$template_file_path = $local_compo_template_dir.$template_dir.DIRECTORY_SEPARATOR.$template_file;
	// Last check - template PHP file
	if(!file_exists($template_file_path) || !is_file($template_file_path) || !is_writable($template_file_path)){
		// No template PHP file
		return 4;
	}

	// Here we are sure that
	// bitrix/templates/<template>/components/bitrix/<component>/<template>/<file>.php
	// exists and writable

	// Try to get template PHP file content
	$template_content = file_get_contents($template_file_path);
	if($template_content === FALSE){
		// cannot read from template PHP file
		return 5;
	}

	// Check is it parsable
	if(!preg_match($pattern, $template_content) === 1){
		// cannot find pattern for addon inserting in template PHP file
		return 6;
	}

	// First clean all previous CLEANTALK template addons
	$ct_template_addon_begin = '<!-- ' . $ct_template_addon_tag . ' -->';	// don't change this!
	$ct_template_addon_end   = '<!-- /' . $ct_template_addon_tag . ' -->';	// don't change this!

	$pos_begin = strpos($template_content, $ct_template_addon_begin);
	$pos_end   = strpos($template_content, $ct_template_addon_end);

	if($pos_begin !== FALSE && $pos_end === FALSE){
		// cannot parse template PHP file
		return 7;
	}elseif($pos_begin === FALSE && $pos_end !== FALSE){
		// cannot parse template PHP file
		return 8;
	}elseif($pos_begin !== FALSE && $pos_end !== FALSE){
		if($pos_begin < $pos_end){
			// cleaning needed
			$template_content = substr($template_content, 0, $pos_begin) . substr($template_content, $pos_end + strlen($ct_template_addon_end));
		}else{
			// cannot parse template PHP file
			return 9;
		}
	//}elseif($pos_begin === FALSE && $pos_end === FALSE){
	//	// nothing to clean
	}

	// Second add current CLEANTALK template addon

	$ct_template_addon = $ct_template_addon_begin . $ct_template_addon_body . $ct_template_addon_end . "\n\n";

	$template_content = preg_replace($pattern, $ct_template_addon . '${1}', $template_content, 1);

	if(!file_put_contents($local_compo_template_dir.$template_dir.DIRECTORY_SEPARATOR.$template_file, $template_content)){
		// Cannot write new content to template PHP file
		return 10;
	}

	// Here all is OK - new template PHP file with CLEANTALK addon inserted is ready
	return 0;
    }

    /**
     * Remove addon from needed local component template
     *
     * @param 	&string $template_dir			Name of component's template dir (.default)
     * @param 	&string $template_file			Name of component's template file (template.php)
     * @param 	&string $local_compo_template_dir	Full local dir of component template (.../bitrix/templates/.default/components/bitrix/system.auth.registration/)
     * @param 	&string $ct_template_addon_tag		Tag string to mark CleanTalk addon body
     * @return 	int Returns error code or 0 when success
     */
    private function uninstall_ct_template($template_dir,	// without ending slash
			     $template_file,
			     $local_compo_template_dir,		// with ending slash
			     $ct_template_addon_tag)
    {
	$template_file_path = $local_compo_template_dir.$template_dir.DIRECTORY_SEPARATOR.$template_file;
	// Last check - template PHP file
	if(!file_exists($template_file_path) || !is_file($template_file_path) || !is_writable($template_file_path)){
		// No template PHP file
		return 4;
	}

	// Here we are sure that
	// bitrix/templates/<template>/components/bitrix/<component>/<template>/<file>.php
	// exists and writable

	// Try to get template PHP file content
	$template_content = file_get_contents($template_file_path);
	if($template_content === FALSE){
		// cannot read from template PHP file
		return 5;
	}

	// Clean all CLEANTALK template addons
	$ct_template_addon_begin = '<!-- ' . $ct_template_addon_tag . ' -->';	// don't change this!
	$ct_template_addon_end   = '<!-- /' . $ct_template_addon_tag . ' -->';	// don't change this!

	$pos_begin = strpos($template_content, $ct_template_addon_begin);
	$pos_end   = strpos($template_content, $ct_template_addon_end);

	if($pos_begin !== FALSE && $pos_end === FALSE){
		// cannot parse template PHP file
		return 7;
	}elseif($pos_begin === FALSE && $pos_end !== FALSE){
		// cannot parse template PHP file
		return 8;
	}elseif($pos_begin !== FALSE && $pos_end !== FALSE){
		if($pos_begin < $pos_end){
			// cleaning needed
			$template_content = substr($template_content, 0, $pos_begin) . substr($template_content, $pos_end + strlen($ct_template_addon_end));
		}else{
			// cannot parse template PHP file
			return 9;
		}
	//}elseif($pos_begin === FALSE && $pos_end === FALSE){
	//	// nothing to clean
	}

	if(!file_put_contents($local_compo_template_dir.$template_dir.DIRECTORY_SEPARATOR.$template_file, $template_content)){
		// Cannot write new content to template PHP file
		return 10;
	}

	// Here all is OK - new template PHP file without any CLEANTALK addon is ready
	return 0;
    }
}
?>
